from tenacity import retry, wait_fixed, stop_after_delay
import trio


@retry(sleep=trio.sleep, wait=wait_fixed(1), stop=stop_after_delay(15))
async def connect_rumor(rumor, enr):
    await rumor.host.start()
    peer_id = await rumor.peer.connect(enr).peer_id()
    return peer_id



from sclients.runner import (
    start_instance, stop_instance, SUPPORTED_CLIENTS
)

from sclients.types import (
    ENR, InstanceConfig
)

from sclients.rumor import (
    connect_rumor
)

__all__ = [
    'start_instance',
    'stop_instance',
    'SUPPORTED_CLIENTS',
    'ENR',
    'InstanceConfig',
    'connect_rumor'
]



from typing import List, NamedTuple


class ENR(NamedTuple):
    private_key: str
    tcp: int
    udp: int
    id: str
    ip: int
    enr: str
    attnets: str
    eth2: str


# NOTE: could turn into 'Instance' to represent a thing that can be on and turned off
class InstanceConfig(NamedTuple):
    client: str
    beacon_state_path: str
    enr: ENR

    @classmethod
    def from_dict(cls, d):
        return cls(d['client'], d['beacon_state_path'], ENR(**d['enr']))
